/****************************************************************************
** Meta object code from reading C++ file 'dbusmenuexporterdbus_p.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.1)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../../kstatusnotifieritem-6.28.0/src/libdbusmenu-qt/dbusmenuexporterdbus_p.h"
#include <QtCore/qmetatype.h>
#include <QtCore/QList>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'dbusmenuexporterdbus_p.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.1. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN20DBusMenuExporterDBusE_t {};
} // unnamed namespace

template <> constexpr inline auto DBusMenuExporterDBus::qt_create_metaobjectdata<qt_meta_tag_ZN20DBusMenuExporterDBusE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "DBusMenuExporterDBus",
        "D-Bus Interface",
        "com.canonical.dbusmenu",
        "ItemsPropertiesUpdated",
        "",
        "DBusMenuItemList",
        "DBusMenuItemKeysList",
        "LayoutUpdated",
        "revision",
        "parentId",
        "ItemActivationRequested",
        "id",
        "timeStamp",
        "Event",
        "Q_NOREPLY",
        "eventId",
        "QDBusVariant",
        "data",
        "timestamp",
        "GetProperty",
        "property",
        "GetLayout",
        "recursionDepth",
        "propertyNames",
        "DBusMenuLayoutItem&",
        "item",
        "GetGroupProperties",
        "QList<int>",
        "ids",
        "AboutToShow",
        "Version",
        "Status"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'ItemsPropertiesUpdated'
        QtMocHelpers::SignalData<void(DBusMenuItemList, DBusMenuItemKeysList)>(3, 4, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 5, 4 }, { 0x80000000 | 6, 4 },
        }}),
        // Signal 'LayoutUpdated'
        QtMocHelpers::SignalData<void(uint, int)>(7, 4, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::UInt, 8 }, { QMetaType::Int, 9 },
        }}),
        // Signal 'ItemActivationRequested'
        QtMocHelpers::SignalData<void(int, uint)>(10, 4, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 11 }, { QMetaType::UInt, 12 },
        }}),
        // Slot 'Event'
        QtMocHelpers::SlotData<void(int, const QString &, const QDBusVariant &, uint)>(13, 14, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 11 }, { QMetaType::QString, 15 }, { 0x80000000 | 16, 17 }, { QMetaType::UInt, 18 },
        }}),
        // Slot 'GetProperty'
        QtMocHelpers::SlotData<QDBusVariant(int, const QString &)>(19, 4, QMC::AccessPublic, 0x80000000 | 16, {{
            { QMetaType::Int, 11 }, { QMetaType::QString, 20 },
        }}),
        // Slot 'GetLayout'
        QtMocHelpers::SlotData<uint(int, int, const QStringList &, DBusMenuLayoutItem &)>(21, 4, QMC::AccessPublic, QMetaType::UInt, {{
            { QMetaType::Int, 9 }, { QMetaType::Int, 22 }, { QMetaType::QStringList, 23 }, { 0x80000000 | 24, 25 },
        }}),
        // Slot 'GetGroupProperties'
        QtMocHelpers::SlotData<DBusMenuItemList(const QList<int> &, const QStringList &)>(26, 4, QMC::AccessPublic, 0x80000000 | 5, {{
            { 0x80000000 | 27, 28 }, { QMetaType::QStringList, 23 },
        }}),
        // Slot 'AboutToShow'
        QtMocHelpers::SlotData<bool(int)>(29, 4, QMC::AccessPublic, QMetaType::Bool, {{
            { QMetaType::Int, 11 },
        }}),
    };
    QtMocHelpers::UintData qt_properties {
        // property 'Version'
        QtMocHelpers::PropertyData<uint>(30, QMetaType::UInt, QMC::DefaultPropertyFlags),
        // property 'Status'
        QtMocHelpers::PropertyData<QString>(31, QMetaType::QString, QMC::DefaultPropertyFlags),
    };
    QtMocHelpers::UintData qt_enums {
    };
    QtMocHelpers::UintData qt_constructors {};
    QtMocHelpers::ClassInfos qt_classinfo({
            {    1,    2 },
    });
    return QtMocHelpers::metaObjectData<DBusMenuExporterDBus, qt_meta_tag_ZN20DBusMenuExporterDBusE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums, qt_constructors, qt_classinfo);
}
Q_CONSTINIT const QMetaObject DBusMenuExporterDBus::staticMetaObject = { {
    QMetaObject::SuperData::link<QObject::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN20DBusMenuExporterDBusE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN20DBusMenuExporterDBusE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN20DBusMenuExporterDBusE_t>.metaTypes,
    nullptr
} };

void DBusMenuExporterDBus::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<DBusMenuExporterDBus *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->ItemsPropertiesUpdated((*reinterpret_cast<std::add_pointer_t<DBusMenuItemList>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<DBusMenuItemKeysList>>(_a[2]))); break;
        case 1: _t->LayoutUpdated((*reinterpret_cast<std::add_pointer_t<uint>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<int>>(_a[2]))); break;
        case 2: _t->ItemActivationRequested((*reinterpret_cast<std::add_pointer_t<int>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<uint>>(_a[2]))); break;
        case 3: _t->Event((*reinterpret_cast<std::add_pointer_t<int>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<QDBusVariant>>(_a[3])),(*reinterpret_cast<std::add_pointer_t<uint>>(_a[4]))); break;
        case 4: { QDBusVariant _r = _t->GetProperty((*reinterpret_cast<std::add_pointer_t<int>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[2])));
            if (_a[0]) *reinterpret_cast<QDBusVariant*>(_a[0]) = std::move(_r); }  break;
        case 5: { uint _r = _t->GetLayout((*reinterpret_cast<std::add_pointer_t<int>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<int>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<QStringList>>(_a[3])),(*reinterpret_cast<std::add_pointer_t<DBusMenuLayoutItem&>>(_a[4])));
            if (_a[0]) *reinterpret_cast<uint*>(_a[0]) = std::move(_r); }  break;
        case 6: { DBusMenuItemList _r = _t->GetGroupProperties((*reinterpret_cast<std::add_pointer_t<QList<int>>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QStringList>>(_a[2])));
            if (_a[0]) *reinterpret_cast<DBusMenuItemList*>(_a[0]) = std::move(_r); }  break;
        case 7: { bool _r = _t->AboutToShow((*reinterpret_cast<std::add_pointer_t<int>>(_a[1])));
            if (_a[0]) *reinterpret_cast<bool*>(_a[0]) = std::move(_r); }  break;
        default: ;
        }
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        switch (_id) {
        default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
        case 0:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 1:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< DBusMenuItemKeysList >(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< DBusMenuItemList >(); break;
            }
            break;
        case 3:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 2:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< QDBusVariant >(); break;
            }
            break;
        case 6:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< QList<int> >(); break;
            }
            break;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (DBusMenuExporterDBus::*)(DBusMenuItemList , DBusMenuItemKeysList )>(_a, &DBusMenuExporterDBus::ItemsPropertiesUpdated, 0))
            return;
        if (QtMocHelpers::indexOfMethod<void (DBusMenuExporterDBus::*)(uint , int )>(_a, &DBusMenuExporterDBus::LayoutUpdated, 1))
            return;
        if (QtMocHelpers::indexOfMethod<void (DBusMenuExporterDBus::*)(int , uint )>(_a, &DBusMenuExporterDBus::ItemActivationRequested, 2))
            return;
    }
    if (_c == QMetaObject::ReadProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 0: *reinterpret_cast<uint*>(_v) = _t->Version(); break;
        case 1: *reinterpret_cast<QString*>(_v) = _t->status(); break;
        default: break;
        }
    }
}

const QMetaObject *DBusMenuExporterDBus::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *DBusMenuExporterDBus::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN20DBusMenuExporterDBusE_t>.strings))
        return static_cast<void*>(this);
    return QObject::qt_metacast(_clname);
}

int DBusMenuExporterDBus::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QObject::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 8)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 8;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 8)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 8;
    }
    if (_c == QMetaObject::ReadProperty || _c == QMetaObject::WriteProperty
            || _c == QMetaObject::ResetProperty || _c == QMetaObject::BindableProperty
            || _c == QMetaObject::RegisterPropertyMetaType) {
        qt_static_metacall(this, _c, _id, _a);
        _id -= 2;
    }
    return _id;
}

// SIGNAL 0
void DBusMenuExporterDBus::ItemsPropertiesUpdated(DBusMenuItemList _t1, DBusMenuItemKeysList _t2)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 0, nullptr, _t1, _t2);
}

// SIGNAL 1
void DBusMenuExporterDBus::LayoutUpdated(uint _t1, int _t2)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 1, nullptr, _t1, _t2);
}

// SIGNAL 2
void DBusMenuExporterDBus::ItemActivationRequested(int _t1, uint _t2)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 2, nullptr, _t1, _t2);
}
QT_WARNING_POP
