/****************************************************************************
** Meta object code from reading C++ file 'kstatusnotifieritem.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.1)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../../kstatusnotifieritem-6.28.0/src/kstatusnotifieritem.h"
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'kstatusnotifieritem.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.1. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN19KStatusNotifierItemE_t {};
} // unnamed namespace

template <> constexpr inline auto KStatusNotifierItem::qt_create_metaobjectdata<qt_meta_tag_ZN19KStatusNotifierItemE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "KStatusNotifierItem",
        "scrollRequested",
        "",
        "delta",
        "Qt::Orientation",
        "orientation",
        "activateRequested",
        "active",
        "QPoint",
        "pos",
        "secondaryActivateRequested",
        "quitRequested",
        "activate",
        "hideAssociatedWindow",
        "serviceChange",
        "name",
        "oldOwner",
        "newOwner",
        "contextMenuAboutToShow",
        "quit",
        "minimizeRestore",
        "legacyWheelEvent",
        "legacyActivated",
        "QSystemTrayIcon::ActivationReason",
        "category",
        "ItemCategory",
        "title",
        "status",
        "ItemStatus",
        "iconName",
        "overlayIconName",
        "attentionIconName",
        "toolTipIconName",
        "toolTipTitle",
        "toolTipSubTitle",
        "Passive",
        "Active",
        "NeedsAttention",
        "ApplicationStatus",
        "Communications",
        "SystemServices",
        "Hardware",
        "Reserved"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'scrollRequested'
        QtMocHelpers::SignalData<void(int, Qt::Orientation)>(1, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 3 }, { 0x80000000 | 4, 5 },
        }}),
        // Signal 'activateRequested'
        QtMocHelpers::SignalData<void(bool, const QPoint &)>(6, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 7 }, { 0x80000000 | 8, 9 },
        }}),
        // Signal 'secondaryActivateRequested'
        QtMocHelpers::SignalData<void(const QPoint &)>(10, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 8, 9 },
        }}),
        // Signal 'quitRequested'
        QtMocHelpers::SignalData<void()>(11, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'activate'
        QtMocHelpers::SlotData<void(const QPoint &)>(12, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 8, 9 },
        }}),
        // Slot 'activate'
        QtMocHelpers::SlotData<void()>(12, 2, QMC::AccessPublic | QMC::MethodCloned, QMetaType::Void),
        // Slot 'hideAssociatedWindow'
        QtMocHelpers::SlotData<void()>(13, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'serviceChange'
        QtMocHelpers::SlotData<void(const QString &, const QString &, const QString &)>(14, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::QString, 15 }, { QMetaType::QString, 16 }, { QMetaType::QString, 17 },
        }}),
        // Slot 'contextMenuAboutToShow'
        QtMocHelpers::SlotData<void()>(18, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'quit'
        QtMocHelpers::SlotData<void()>(19, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'minimizeRestore'
        QtMocHelpers::SlotData<void()>(20, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'legacyWheelEvent'
        QtMocHelpers::SlotData<void(int)>(21, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::Int, 2 },
        }}),
        // Slot 'legacyActivated'
        QtMocHelpers::SlotData<void(QSystemTrayIcon::ActivationReason)>(22, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { 0x80000000 | 23, 2 },
        }}),
    };
    QtMocHelpers::UintData qt_properties {
        // property 'category'
        QtMocHelpers::PropertyData<enum ItemCategory>(24, 0x80000000 | 25, QMC::DefaultPropertyFlags | QMC::Writable | QMC::EnumOrFlag | QMC::StdCppSet),
        // property 'title'
        QtMocHelpers::PropertyData<QString>(26, QMetaType::QString, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet),
        // property 'status'
        QtMocHelpers::PropertyData<enum ItemStatus>(27, 0x80000000 | 28, QMC::DefaultPropertyFlags | QMC::Writable | QMC::EnumOrFlag | QMC::StdCppSet),
        // property 'iconName'
        QtMocHelpers::PropertyData<QString>(29, QMetaType::QString, QMC::DefaultPropertyFlags | QMC::Writable),
        // property 'overlayIconName'
        QtMocHelpers::PropertyData<QString>(30, QMetaType::QString, QMC::DefaultPropertyFlags | QMC::Writable),
        // property 'attentionIconName'
        QtMocHelpers::PropertyData<QString>(31, QMetaType::QString, QMC::DefaultPropertyFlags | QMC::Writable),
        // property 'toolTipIconName'
        QtMocHelpers::PropertyData<QString>(32, QMetaType::QString, QMC::DefaultPropertyFlags | QMC::Writable),
        // property 'toolTipTitle'
        QtMocHelpers::PropertyData<QString>(33, QMetaType::QString, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet),
        // property 'toolTipSubTitle'
        QtMocHelpers::PropertyData<QString>(34, QMetaType::QString, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet),
    };
    QtMocHelpers::UintData qt_enums {
        // enum 'ItemStatus'
        QtMocHelpers::EnumData<enum ItemStatus>(28, 28, QMC::EnumFlags{}).add({
            {   35, ItemStatus::Passive },
            {   36, ItemStatus::Active },
            {   37, ItemStatus::NeedsAttention },
        }),
        // enum 'ItemCategory'
        QtMocHelpers::EnumData<enum ItemCategory>(25, 25, QMC::EnumFlags{}).add({
            {   38, ItemCategory::ApplicationStatus },
            {   39, ItemCategory::Communications },
            {   40, ItemCategory::SystemServices },
            {   41, ItemCategory::Hardware },
            {   42, ItemCategory::Reserved },
        }),
    };
    return QtMocHelpers::metaObjectData<KStatusNotifierItem, qt_meta_tag_ZN19KStatusNotifierItemE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject KStatusNotifierItem::staticMetaObject = { {
    QMetaObject::SuperData::link<QObject::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN19KStatusNotifierItemE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN19KStatusNotifierItemE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN19KStatusNotifierItemE_t>.metaTypes,
    nullptr
} };

void KStatusNotifierItem::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<KStatusNotifierItem *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->scrollRequested((*reinterpret_cast<std::add_pointer_t<int>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<Qt::Orientation>>(_a[2]))); break;
        case 1: _t->activateRequested((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QPoint>>(_a[2]))); break;
        case 2: _t->secondaryActivateRequested((*reinterpret_cast<std::add_pointer_t<QPoint>>(_a[1]))); break;
        case 3: _t->quitRequested(); break;
        case 4: _t->activate((*reinterpret_cast<std::add_pointer_t<QPoint>>(_a[1]))); break;
        case 5: _t->activate(); break;
        case 6: _t->hideAssociatedWindow(); break;
        case 7: _t->d->serviceChange((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[3]))); break;
        case 8: _t->d->contextMenuAboutToShow(); break;
        case 9: _t->d->quit(); break;
        case 10: _t->d->minimizeRestore(); break;
        case 11: _t->d->legacyWheelEvent((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 12: _t->d->legacyActivated((*reinterpret_cast<std::add_pointer_t<QSystemTrayIcon::ActivationReason>>(_a[1]))); break;
        default: ;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (KStatusNotifierItem::*)(int , Qt::Orientation )>(_a, &KStatusNotifierItem::scrollRequested, 0))
            return;
        if (QtMocHelpers::indexOfMethod<void (KStatusNotifierItem::*)(bool , const QPoint & )>(_a, &KStatusNotifierItem::activateRequested, 1))
            return;
        if (QtMocHelpers::indexOfMethod<void (KStatusNotifierItem::*)(const QPoint & )>(_a, &KStatusNotifierItem::secondaryActivateRequested, 2))
            return;
        if (QtMocHelpers::indexOfMethod<void (KStatusNotifierItem::*)()>(_a, &KStatusNotifierItem::quitRequested, 3))
            return;
    }
    if (_c == QMetaObject::ReadProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 0: *reinterpret_cast<enum ItemCategory*>(_v) = _t->category(); break;
        case 1: *reinterpret_cast<QString*>(_v) = _t->title(); break;
        case 2: *reinterpret_cast<enum ItemStatus*>(_v) = _t->status(); break;
        case 3: *reinterpret_cast<QString*>(_v) = _t->iconName(); break;
        case 4: *reinterpret_cast<QString*>(_v) = _t->overlayIconName(); break;
        case 5: *reinterpret_cast<QString*>(_v) = _t->attentionIconName(); break;
        case 6: *reinterpret_cast<QString*>(_v) = _t->toolTipIconName(); break;
        case 7: *reinterpret_cast<QString*>(_v) = _t->toolTipTitle(); break;
        case 8: *reinterpret_cast<QString*>(_v) = _t->toolTipSubTitle(); break;
        default: break;
        }
    }
    if (_c == QMetaObject::WriteProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 0: _t->setCategory(*reinterpret_cast<enum ItemCategory*>(_v)); break;
        case 1: _t->setTitle(*reinterpret_cast<QString*>(_v)); break;
        case 2: _t->setStatus(*reinterpret_cast<enum ItemStatus*>(_v)); break;
        case 3: _t->setIconByName(*reinterpret_cast<QString*>(_v)); break;
        case 4: _t->setOverlayIconByName(*reinterpret_cast<QString*>(_v)); break;
        case 5: _t->setAttentionIconByName(*reinterpret_cast<QString*>(_v)); break;
        case 6: _t->setToolTipIconByName(*reinterpret_cast<QString*>(_v)); break;
        case 7: _t->setToolTipTitle(*reinterpret_cast<QString*>(_v)); break;
        case 8: _t->setToolTipSubTitle(*reinterpret_cast<QString*>(_v)); break;
        default: break;
        }
    }
}

const QMetaObject *KStatusNotifierItem::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *KStatusNotifierItem::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN19KStatusNotifierItemE_t>.strings))
        return static_cast<void*>(this);
    return QObject::qt_metacast(_clname);
}

int KStatusNotifierItem::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QObject::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 13)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 13;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 13)
            *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType();
        _id -= 13;
    }
    if (_c == QMetaObject::ReadProperty || _c == QMetaObject::WriteProperty
            || _c == QMetaObject::ResetProperty || _c == QMetaObject::BindableProperty
            || _c == QMetaObject::RegisterPropertyMetaType) {
        qt_static_metacall(this, _c, _id, _a);
        _id -= 9;
    }
    return _id;
}

// SIGNAL 0
void KStatusNotifierItem::scrollRequested(int _t1, Qt::Orientation _t2)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 0, nullptr, _t1, _t2);
}

// SIGNAL 1
void KStatusNotifierItem::activateRequested(bool _t1, const QPoint & _t2)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 1, nullptr, _t1, _t2);
}

// SIGNAL 2
void KStatusNotifierItem::secondaryActivateRequested(const QPoint & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 2, nullptr, _t1);
}

// SIGNAL 3
void KStatusNotifierItem::quitRequested()
{
    QMetaObject::activate(this, &staticMetaObject, 3, nullptr);
}
QT_WARNING_POP
