/*
 * This file was generated by qdbusxml2cpp version 0.8
 * Source file was com.canonical.dbusmenu.xml
 *
 * qdbusxml2cpp is Copyright (C) The Qt Company Ltd. and other contributors.
 *
 * This is an auto-generated file.
 * This file may have been hand-edited. Look for HAND-EDIT comments
 * before re-generating it.
 */

#ifndef DBUSMENUADAPTOR_H
#define DBUSMENUADAPTOR_H

#include <QtCore/QObject>
#include <QtDBus/QtDBus>
#include "libdbusmenu-qt/dbusmenuexporterdbus_p.h"
#include <QtCore/qcontainerfwd.h>

/*
 * Adaptor class for interface com.canonical.dbusmenu
 */
class DbusmenuAdaptor: public QDBusAbstractAdaptor
{
    Q_OBJECT
    Q_CLASSINFO("D-Bus Interface", "com.canonical.dbusmenu")
    Q_CLASSINFO("D-Bus Introspection", ""
"  <interface name=\"com.canonical.dbusmenu\">\n"
"    <property access=\"read\" type=\"u\" name=\"Version\">\n"
"    </property>\n"
"    <property access=\"read\" type=\"s\" name=\"Status\">\n"
"    </property>\n"
"    <method name=\"GetLayout\">\n"
"      <annotation value=\"DBusMenuLayoutItem\" name=\"org.qtproject.QtDBus.QtTypeName.Out1\"/>\n"
"      <arg direction=\"in\" type=\"i\" name=\"parentId\"/>\n"
"      <arg direction=\"in\" type=\"i\" name=\"recursionDepth\"/>\n"
"      <arg direction=\"in\" type=\"as\" name=\"propertyNames\"/>\n"
"      <arg direction=\"out\" type=\"u\" name=\"revision\"/>\n"
"      <arg direction=\"out\" type=\"(ia{sv}av)\" name=\"layout\"/>\n"
"    </method>\n"
"    <method name=\"GetGroupProperties\">\n"
"      <annotation value=\"QList&lt;int&gt;\" name=\"org.qtproject.QtDBus.QtTypeName.In0\"/>\n"
"      <annotation value=\"DBusMenuItemList\" name=\"org.qtproject.QtDBus.QtTypeName.Out0\"/>\n"
"      <arg direction=\"in\" type=\"ai\" name=\"ids\"/>\n"
"      <arg direction=\"in\" type=\"as\" name=\"propertyNames\"/>\n"
"      <arg direction=\"out\" type=\"a(ia{sv})\" name=\"properties\"/>\n"
"    </method>\n"
"    <method name=\"GetProperty\">\n"
"      <arg direction=\"in\" type=\"i\" name=\"id\"/>\n"
"      <arg direction=\"in\" type=\"s\" name=\"name\"/>\n"
"      <arg direction=\"out\" type=\"v\" name=\"value\"/>\n"
"    </method>\n"
"    <method name=\"Event\">\n"
"      <arg direction=\"in\" type=\"i\" name=\"id\"/>\n"
"      <arg direction=\"in\" type=\"s\" name=\"eventId\"/>\n"
"      <arg direction=\"in\" type=\"v\" name=\"data\"/>\n"
"      <arg direction=\"in\" type=\"u\" name=\"timestamp\"/>\n"
"    </method>\n"
"    <method name=\"AboutToShow\">\n"
"      <arg direction=\"in\" type=\"i\" name=\"id\"/>\n"
"      <arg direction=\"out\" type=\"b\" name=\"needUpdate\"/>\n"
"    </method>\n"
"    <signal name=\"ItemsPropertiesUpdated\">\n"
"      <annotation value=\"DBusMenuItemList\" name=\"org.qtproject.QtDBus.QtTypeName.Out0\"/>\n"
"      <annotation value=\"DBusMenuItemKeysList\" name=\"org.qtproject.QtDBus.QtTypeName.Out1\"/>\n"
"      <arg direction=\"out\" type=\"a(ia{sv})\" name=\"updatedProps\"/>\n"
"      <arg direction=\"out\" type=\"a(ias)\" name=\"removedProps\"/>\n"
"    </signal>\n"
"    <signal name=\"LayoutUpdated\">\n"
"      <arg direction=\"out\" type=\"u\" name=\"revision\"/>\n"
"      <arg direction=\"out\" type=\"i\" name=\"parent\"/>\n"
"    </signal>\n"
"    <signal name=\"ItemActivationRequested\">\n"
"      <arg direction=\"out\" type=\"i\" name=\"id\"/>\n"
"      <arg direction=\"out\" type=\"u\" name=\"timestamp\"/>\n"
"    </signal>\n"
"  </interface>\n"
        "")
public:
    DbusmenuAdaptor(DBusMenuExporterDBus *parent);
    ~DbusmenuAdaptor() override;

    inline DBusMenuExporterDBus *parent() const
    { return static_cast<DBusMenuExporterDBus *>(QObject::parent()); }

public: // PROPERTIES
    Q_PROPERTY(QString Status READ status)
    QString status() const;

    Q_PROPERTY(uint Version READ version)
    uint version() const;

public Q_SLOTS: // METHODS
    bool AboutToShow(int id);
    void Event(int id, const QString &eventId, const QDBusVariant &data, uint timestamp);
    DBusMenuItemList GetGroupProperties(const QList<int> &ids, const QStringList &propertyNames);
    uint GetLayout(int parentId, int recursionDepth, const QStringList &propertyNames, DBusMenuLayoutItem &layout);
    QDBusVariant GetProperty(int id, const QString &name);
Q_SIGNALS: // SIGNALS
    void ItemActivationRequested(int id, uint timestamp);
    void ItemsPropertiesUpdated(DBusMenuItemList updatedProps, DBusMenuItemKeysList removedProps);
    void LayoutUpdated(uint revision, int parent);
};

#endif
