/****************************************************************************
** Meta object code from reading C++ file 'kstatusnotifieritemdbus_p.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.1)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../../kstatusnotifieritem-6.28.0/src/kstatusnotifieritemdbus_p.h"
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'kstatusnotifieritemdbus_p.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.1. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN23KStatusNotifierItemDBusE_t {};
} // unnamed namespace

template <> constexpr inline auto KStatusNotifierItemDBus::qt_create_metaobjectdata<qt_meta_tag_ZN23KStatusNotifierItemDBusE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "KStatusNotifierItemDBus",
        "NewIcon",
        "",
        "NewOverlayIcon",
        "NewAttentionIcon",
        "NewMenu",
        "NewToolTip",
        "NewStatus",
        "status",
        "ContextMenu",
        "x",
        "y",
        "Activate",
        "SecondaryActivate",
        "Scroll",
        "delta",
        "orientation",
        "ProvideXdgActivationToken",
        "token",
        "Category",
        "Id",
        "Title",
        "Status",
        "WindowId",
        "ItemIsMenu",
        "IconName",
        "IconPixmap",
        "KDbusImageVector",
        "OverlayIconName",
        "OverlayIconPixmap",
        "AttentionIconName",
        "AttentionIconPixmap",
        "AttentionMovieName",
        "ToolTip",
        "KDbusToolTipStruct",
        "IconThemePath",
        "Menu",
        "QDBusObjectPath"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'NewIcon'
        QtMocHelpers::SignalData<void()>(1, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'NewOverlayIcon'
        QtMocHelpers::SignalData<void()>(3, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'NewAttentionIcon'
        QtMocHelpers::SignalData<void()>(4, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'NewMenu'
        QtMocHelpers::SignalData<void()>(5, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'NewToolTip'
        QtMocHelpers::SignalData<void()>(6, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'NewStatus'
        QtMocHelpers::SignalData<void(const QString &)>(7, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 8 },
        }}),
        // Slot 'ContextMenu'
        QtMocHelpers::SlotData<void(int, int)>(9, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 10 }, { QMetaType::Int, 11 },
        }}),
        // Slot 'Activate'
        QtMocHelpers::SlotData<void(int, int)>(12, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 10 }, { QMetaType::Int, 11 },
        }}),
        // Slot 'SecondaryActivate'
        QtMocHelpers::SlotData<void(int, int)>(13, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 10 }, { QMetaType::Int, 11 },
        }}),
        // Slot 'Scroll'
        QtMocHelpers::SlotData<void(int, const QString &)>(14, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 15 }, { QMetaType::QString, 16 },
        }}),
        // Slot 'ProvideXdgActivationToken'
        QtMocHelpers::SlotData<void(const QString &)>(17, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 18 },
        }}),
    };
    QtMocHelpers::UintData qt_properties {
        // property 'Category'
        QtMocHelpers::PropertyData<QString>(19, QMetaType::QString, QMC::DefaultPropertyFlags),
        // property 'Id'
        QtMocHelpers::PropertyData<QString>(20, QMetaType::QString, QMC::DefaultPropertyFlags),
        // property 'Title'
        QtMocHelpers::PropertyData<QString>(21, QMetaType::QString, QMC::DefaultPropertyFlags),
        // property 'Status'
        QtMocHelpers::PropertyData<QString>(22, QMetaType::QString, QMC::DefaultPropertyFlags),
        // property 'WindowId'
        QtMocHelpers::PropertyData<int>(23, QMetaType::Int, QMC::DefaultPropertyFlags),
        // property 'ItemIsMenu'
        QtMocHelpers::PropertyData<bool>(24, QMetaType::Bool, QMC::DefaultPropertyFlags),
        // property 'IconName'
        QtMocHelpers::PropertyData<QString>(25, QMetaType::QString, QMC::DefaultPropertyFlags),
        // property 'IconPixmap'
        QtMocHelpers::PropertyData<KDbusImageVector>(26, 0x80000000 | 27, QMC::DefaultPropertyFlags | QMC::EnumOrFlag),
        // property 'OverlayIconName'
        QtMocHelpers::PropertyData<QString>(28, QMetaType::QString, QMC::DefaultPropertyFlags),
        // property 'OverlayIconPixmap'
        QtMocHelpers::PropertyData<KDbusImageVector>(29, 0x80000000 | 27, QMC::DefaultPropertyFlags | QMC::EnumOrFlag),
        // property 'AttentionIconName'
        QtMocHelpers::PropertyData<QString>(30, QMetaType::QString, QMC::DefaultPropertyFlags),
        // property 'AttentionIconPixmap'
        QtMocHelpers::PropertyData<KDbusImageVector>(31, 0x80000000 | 27, QMC::DefaultPropertyFlags | QMC::EnumOrFlag),
        // property 'AttentionMovieName'
        QtMocHelpers::PropertyData<QString>(32, QMetaType::QString, QMC::DefaultPropertyFlags),
        // property 'ToolTip'
        QtMocHelpers::PropertyData<KDbusToolTipStruct>(33, 0x80000000 | 34, QMC::DefaultPropertyFlags | QMC::EnumOrFlag),
        // property 'IconThemePath'
        QtMocHelpers::PropertyData<QString>(35, QMetaType::QString, QMC::DefaultPropertyFlags),
        // property 'Menu'
        QtMocHelpers::PropertyData<QDBusObjectPath>(36, 0x80000000 | 37, QMC::DefaultPropertyFlags | QMC::EnumOrFlag),
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<KStatusNotifierItemDBus, qt_meta_tag_ZN23KStatusNotifierItemDBusE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject KStatusNotifierItemDBus::staticMetaObject = { {
    QMetaObject::SuperData::link<QObject::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN23KStatusNotifierItemDBusE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN23KStatusNotifierItemDBusE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN23KStatusNotifierItemDBusE_t>.metaTypes,
    nullptr
} };

void KStatusNotifierItemDBus::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<KStatusNotifierItemDBus *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->NewIcon(); break;
        case 1: _t->NewOverlayIcon(); break;
        case 2: _t->NewAttentionIcon(); break;
        case 3: _t->NewMenu(); break;
        case 4: _t->NewToolTip(); break;
        case 5: _t->NewStatus((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 6: _t->ContextMenu((*reinterpret_cast<std::add_pointer_t<int>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<int>>(_a[2]))); break;
        case 7: _t->Activate((*reinterpret_cast<std::add_pointer_t<int>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<int>>(_a[2]))); break;
        case 8: _t->SecondaryActivate((*reinterpret_cast<std::add_pointer_t<int>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<int>>(_a[2]))); break;
        case 9: _t->Scroll((*reinterpret_cast<std::add_pointer_t<int>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[2]))); break;
        case 10: _t->ProvideXdgActivationToken((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        default: ;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (KStatusNotifierItemDBus::*)()>(_a, &KStatusNotifierItemDBus::NewIcon, 0))
            return;
        if (QtMocHelpers::indexOfMethod<void (KStatusNotifierItemDBus::*)()>(_a, &KStatusNotifierItemDBus::NewOverlayIcon, 1))
            return;
        if (QtMocHelpers::indexOfMethod<void (KStatusNotifierItemDBus::*)()>(_a, &KStatusNotifierItemDBus::NewAttentionIcon, 2))
            return;
        if (QtMocHelpers::indexOfMethod<void (KStatusNotifierItemDBus::*)()>(_a, &KStatusNotifierItemDBus::NewMenu, 3))
            return;
        if (QtMocHelpers::indexOfMethod<void (KStatusNotifierItemDBus::*)()>(_a, &KStatusNotifierItemDBus::NewToolTip, 4))
            return;
        if (QtMocHelpers::indexOfMethod<void (KStatusNotifierItemDBus::*)(const QString & )>(_a, &KStatusNotifierItemDBus::NewStatus, 5))
            return;
    }
    if (_c == QMetaObject::RegisterPropertyMetaType) {
        switch (_id) {
        default: *reinterpret_cast<int*>(_a[0]) = -1; break;
        case 11:
        case 9:
        case 7:
            *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< KDbusImageVector >(); break;
        case 13:
            *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< KDbusToolTipStruct >(); break;
        case 15:
            *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< QDBusObjectPath >(); break;
        }
    }
    if (_c == QMetaObject::ReadProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 0: *reinterpret_cast<QString*>(_v) = _t->Category(); break;
        case 1: *reinterpret_cast<QString*>(_v) = _t->Id(); break;
        case 2: *reinterpret_cast<QString*>(_v) = _t->Title(); break;
        case 3: *reinterpret_cast<QString*>(_v) = _t->Status(); break;
        case 4: *reinterpret_cast<int*>(_v) = _t->WindowId(); break;
        case 5: *reinterpret_cast<bool*>(_v) = _t->ItemIsMenu(); break;
        case 6: *reinterpret_cast<QString*>(_v) = _t->IconName(); break;
        case 7: *reinterpret_cast<KDbusImageVector*>(_v) = _t->IconPixmap(); break;
        case 8: *reinterpret_cast<QString*>(_v) = _t->OverlayIconName(); break;
        case 9: *reinterpret_cast<KDbusImageVector*>(_v) = _t->OverlayIconPixmap(); break;
        case 10: *reinterpret_cast<QString*>(_v) = _t->AttentionIconName(); break;
        case 11: *reinterpret_cast<KDbusImageVector*>(_v) = _t->AttentionIconPixmap(); break;
        case 12: *reinterpret_cast<QString*>(_v) = _t->AttentionMovieName(); break;
        case 13: *reinterpret_cast<KDbusToolTipStruct*>(_v) = _t->ToolTip(); break;
        case 14: *reinterpret_cast<QString*>(_v) = _t->IconThemePath(); break;
        case 15: *reinterpret_cast<QDBusObjectPath*>(_v) = _t->Menu(); break;
        default: break;
        }
    }
}

const QMetaObject *KStatusNotifierItemDBus::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *KStatusNotifierItemDBus::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN23KStatusNotifierItemDBusE_t>.strings))
        return static_cast<void*>(this);
    return QObject::qt_metacast(_clname);
}

int KStatusNotifierItemDBus::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QObject::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 11)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 11;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 11)
            *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType();
        _id -= 11;
    }
    if (_c == QMetaObject::ReadProperty || _c == QMetaObject::WriteProperty
            || _c == QMetaObject::ResetProperty || _c == QMetaObject::BindableProperty
            || _c == QMetaObject::RegisterPropertyMetaType) {
        qt_static_metacall(this, _c, _id, _a);
        _id -= 16;
    }
    return _id;
}

// SIGNAL 0
void KStatusNotifierItemDBus::NewIcon()
{
    QMetaObject::activate(this, &staticMetaObject, 0, nullptr);
}

// SIGNAL 1
void KStatusNotifierItemDBus::NewOverlayIcon()
{
    QMetaObject::activate(this, &staticMetaObject, 1, nullptr);
}

// SIGNAL 2
void KStatusNotifierItemDBus::NewAttentionIcon()
{
    QMetaObject::activate(this, &staticMetaObject, 2, nullptr);
}

// SIGNAL 3
void KStatusNotifierItemDBus::NewMenu()
{
    QMetaObject::activate(this, &staticMetaObject, 3, nullptr);
}

// SIGNAL 4
void KStatusNotifierItemDBus::NewToolTip()
{
    QMetaObject::activate(this, &staticMetaObject, 4, nullptr);
}

// SIGNAL 5
void KStatusNotifierItemDBus::NewStatus(const QString & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 5, nullptr, _t1);
}
QT_WARNING_POP
