

#ifndef SBK_KSTATUSNOTIFIERITEM_PYTHON_H
#define SBK_KSTATUSNOTIFIERITEM_PYTHON_H

//workaround to access protected functions
#define protected public

#include <sbkpython.h>
#include <sbkmodule.h>
#include <sbkconverter.h>
// Module Includes
#include <pyside6_qtcore_python.h>
#include <pyside6_qtgui_python.h>
#include <pyside6_qtwidgets_python.h>

// Bound library includes
#include <kstatusnotifieritem.h>
// Conversion Includes - Primitive Types
#include <QAnyStringView>
#include <qbytearrayview.h>
#include <qchar.h>
#include <QtCore/qlatin1stringview.h>
#include <QString>
#include <QStringList>
#include <QStringView>
#include <qvariant.h>

// Conversion Includes - Container Types
#include <QHash>
#include <QList>
#include <QMap>
#include <QMultiHash>
#include <QMultiMap>
#include <QPair>
#include <QQueue>
#include <QSet>
#include <QStack>
#include <array>
#include <list>
#include <map>
#include <utility>
#include <unordered_map>
#include <vector>


// Type indices
enum : int {
    SBK_KStatusNotifierItem_ItemStatus_IDX                   = 2,
    SBK_KStatusNotifierItem_ItemCategory_IDX                 = 1,
    SBK_KStatusNotifierItem_IDX                              = 0,
    SBK_KStatusNotifierItem_IDX_COUNT                        = 3,
};

// This variable stores all Python types exported by this module.
extern Shiboken::Module::TypeInitStruct *SbkKStatusNotifierItemTypeStructs;

// This variable stores the Python module object exported by this module.
extern PyObject *SbkKStatusNotifierItemModuleObject;

// This variable stores all type converters exported by this module.
extern SbkConverter **SbkKStatusNotifierItemTypeConverters;

// Converter indices
enum [[deprecated]] : int {
    SBK_KSTATUSNOTIFIERITEM_QLIST_INT_IDX                    = 0, // QList<int>
    SBK_KSTATUSNOTIFIERITEM_QLIST_QACTIONPTR_IDX             = 1, // QList<QAction*>
    SBK_KSTATUSNOTIFIERITEM_QLIST_QVARIANT_IDX               = 2, // QList<QVariant>
    SBK_KSTATUSNOTIFIERITEM_QLIST_QSTRING_IDX                = 3, // QList<QString>
    SBK_KSTATUSNOTIFIERITEM_QMAP_QSTRING_QVARIANT_IDX        = 4, // QMap<QString,QVariant>
    SBK_KSTATUSNOTIFIERITEM_CONVERTERS_IDX_COUNT             = 5,
};

// Converter indices
enum : int {
    SBK_KStatusNotifierItem_QList_int_IDX                    = 0, // QList<int>
    SBK_KStatusNotifierItem_QList_QActionPTR_IDX             = 1, // QList<QAction*>
    SBK_KStatusNotifierItem_QList_QVariant_IDX               = 2, // QList<QVariant>
    SBK_KStatusNotifierItem_QList_QString_IDX                = 3, // QList<QString>
    SBK_KStatusNotifierItem_QMap_QString_QVariant_IDX        = 4, // QMap<QString,QVariant>
    SBK_KStatusNotifierItem_CONVERTERS_IDX_COUNT             = 5,
};
// Macros for type check

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
namespace Shiboken
{

// PyType functions, to get the PyObjectType for a type T
template<> inline PyTypeObject *SbkType< ::KStatusNotifierItem::ItemStatus >() { return Shiboken::Module::get(SbkKStatusNotifierItemTypeStructs[SBK_KStatusNotifierItem_ItemStatus_IDX]); }
template<> inline PyTypeObject *SbkType< ::KStatusNotifierItem::ItemCategory >() { return Shiboken::Module::get(SbkKStatusNotifierItemTypeStructs[SBK_KStatusNotifierItem_ItemCategory_IDX]); }
template<> inline PyTypeObject *SbkType< ::KStatusNotifierItem >() { return Shiboken::Module::get(SbkKStatusNotifierItemTypeStructs[SBK_KStatusNotifierItem_IDX]); }

} // namespace Shiboken

QT_WARNING_POP
#endif // SBK_KSTATUSNOTIFIERITEM_PYTHON_H

